""" Utilities for passing arguments to datafetcher in tests """
import os
from pathlib import Path
from typing import List

from datafetcher.errors import MissingSettingsException, NoProfileException
from datafetcher.utilities import get_arg_parser, path_is_accessible


def parse_args(args: List[str]):
    """
    Wrapper around parser.parse_args() so in the event of a foo
    an exception is thrown rather than sys.exe

    :param args:
    :return:
    """
    confirm_complete_args(args)
    get_arg_parser().parse_args(args)


def confirm_complete_args(args: List[str]):
    """
    Let's scrutinize the args -before- calling parse_args()
    so we can differentiate among errors.

    :param args:
    :return:
    """

    # we must have a profile
    assert "--profile" in args or "CAPO_PROFILE" in os.environ
    if "--profile" not in args and "CAPO_PROFILE" in os.environ:
        raise NoProfileException("Capo profile is required.")

    # we must have an output dir....
    assert "--output-dir" in args

    # ... and it must exist
    args_iter = iter(args)
    args_dict = dict(zip(args_iter, args_iter))

    output_dir_arg = args_dict["--output-dir"]
    output_dir = Path(output_dir_arg)
    if not output_dir.exists():
        # did they just forget to specify it?
        if output_dir_arg.startswith("--"):
            raise MissingSettingsException("--output-dir is required")
        raise FileNotFoundError(f"output dir '{output_dir_arg}' not found")
    if not path_is_accessible(output_dir):
        raise FileNotFoundError(f"permission denied on {output_dir}")

    # we must have EITHER a product locator OR a locations file....
    if "--product-locator" not in args_dict.keys() and "--location-file" not in args_dict.keys():
        raise MissingSettingsException("either product locator or locations file required")
    if "--product-locator" in args_dict.keys() and "--location-file" in args_dict.keys():
        raise MissingSettingsException("product locator OR locations file required -- not both")

    if "--product-locator" in args:
        product_locator = args_dict["--product-locator"]
        assert product_locator
    else:
        locations_file = args_dict["--location-file"]
        if not Path(locations_file).exists():
            raise FileNotFoundError(f"{locations_file} not found")
